import { isUndefined } from 'lodash';
import { useEffect, useRef } from 'react';
import { fromEvent } from 'rxjs';
import { useAsync, useEvent, useRefExtra, useResize } from '@react-devui/hooks';
import { cloneHTMLElement } from '../../utils';
import { useGlobalScroll, useLayout } from '../root';
export function DPopup(props) {
    const { children, dVisible, dTrigger, dMouseEnterDelay = 150, dMouseLeaveDelay = 200, dUpdatePosition, onVisibleChange } = props;
    //#region Context
    const { dPageScrollRef, dContentResizeRef } = useLayout();
    //#endregion
    //#region Ref
    const windowRef = useRefExtra(() => window);
    //#endregion
    const dataRef = useRef({});
    const async = useAsync();
    const handleTrigger = (visible, behavior) => {
        var _a, _b;
        (_b = (_a = dataRef.current).clearTid) === null || _b === void 0 ? void 0 : _b.call(_a);
        const changeVisible = () => {
            if (isUndefined(visible)) {
                onVisibleChange(!dVisible);
            }
            else if (!Object.is(dVisible, visible)) {
                onVisibleChange(visible);
            }
        };
        if (behavior) {
            switch (behavior) {
                case 'hover':
                    if (visible) {
                        dataRef.current.clearTid = async.setTimeout(() => changeVisible(), dMouseEnterDelay);
                    }
                    else {
                        dataRef.current.clearTid = async.setTimeout(() => changeVisible(), dMouseLeaveDelay);
                    }
                    break;
                case 'popup-hover':
                    if (!visible) {
                        dataRef.current.clearTid = async.setTimeout(() => changeVisible(), dMouseLeaveDelay);
                    }
                    break;
                default:
                    break;
            }
        }
        else {
            changeVisible();
        }
    };
    const globalScroll = useGlobalScroll(dUpdatePosition.fn, !dVisible);
    useEffect(() => {
        if (dVisible && !globalScroll) {
            const ob = fromEvent([dPageScrollRef, ...dUpdatePosition.extraScrollRefs].map((ref) => ref === null || ref === void 0 ? void 0 : ref.current).filter((el) => el), 'scroll', { passive: true }).subscribe({
                next: () => {
                    dUpdatePosition.fn();
                },
            });
            return () => {
                ob.unsubscribe();
            };
        }
    });
    useResize(dUpdatePosition.triggerRef, dUpdatePosition.fn, !dVisible);
    useResize(dUpdatePosition.popupRef, dUpdatePosition.fn, !dVisible);
    useResize(dContentResizeRef, dUpdatePosition.fn, !dVisible);
    useEvent(windowRef, 'click', () => {
        dataRef.current.clearClickOut = async.requestAnimationFrame(() => {
            handleTrigger(false);
        });
    }, { capture: true }, !dVisible || dTrigger !== 'click');
    return children({
        renderTrigger: (el) => {
            const triggerProps = {};
            switch (dTrigger) {
                case 'hover':
                    triggerProps.onMouseEnter = (e) => {
                        var _a, _b;
                        (_b = (_a = el.props).onMouseEnter) === null || _b === void 0 ? void 0 : _b.call(_a, e);
                        handleTrigger(true, 'hover');
                    };
                    triggerProps.onMouseLeave = (e) => {
                        var _a, _b;
                        (_b = (_a = el.props).onMouseLeave) === null || _b === void 0 ? void 0 : _b.call(_a, e);
                        handleTrigger(false, 'hover');
                    };
                    break;
                case 'click':
                    triggerProps.onClick = (e) => {
                        var _a, _b, _c, _d;
                        (_b = (_a = el.props).onClick) === null || _b === void 0 ? void 0 : _b.call(_a, e);
                        (_d = (_c = dataRef.current).clearClickOut) === null || _d === void 0 ? void 0 : _d.call(_c);
                        handleTrigger();
                    };
                    break;
                default:
                    break;
            }
            return cloneHTMLElement(el, triggerProps);
        },
        renderPopup: (el) => {
            const popupProps = {};
            if (dVisible) {
                switch (dTrigger) {
                    case 'hover':
                        popupProps.onMouseEnter = (e) => {
                            var _a, _b;
                            (_b = (_a = el.props).onMouseEnter) === null || _b === void 0 ? void 0 : _b.call(_a, e);
                            handleTrigger(true, 'popup-hover');
                        };
                        popupProps.onMouseLeave = (e) => {
                            var _a, _b;
                            (_b = (_a = el.props).onMouseLeave) === null || _b === void 0 ? void 0 : _b.call(_a, e);
                            handleTrigger(false, 'popup-hover');
                        };
                        break;
                    case 'click':
                        popupProps.onClick = (e) => {
                            var _a, _b, _c, _d;
                            (_b = (_a = el.props).onClick) === null || _b === void 0 ? void 0 : _b.call(_a, e);
                            (_d = (_c = dataRef.current).clearClickOut) === null || _d === void 0 ? void 0 : _d.call(_c);
                        };
                        break;
                    default:
                        break;
                }
            }
            return cloneHTMLElement(el, popupProps);
        },
    });
}
